using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.IO;
using System.Data;
using System.Net;
using System.Net.Sockets;
using System.Diagnostics;
using System.Runtime.InteropServices;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters.Binary;


namespace WindowsApplication1
{
	public delegate void DelegateAddMemInfo(MemInfo i);
    public delegate void DelegateRemoveMemInfo(uint addr);
    public delegate void DelegateUpdateStats();
    public delegate void DelegateAssertion(String expr, uint[] callStack);

	/// <summary>
	/// Summary description for Form1.
	/// </summary>
	public class Form1 : System.Windows.Forms.Form
	{
		private System.ComponentModel.IContainer components;
        private System.Windows.Forms.TabControl tabControl1;
		private System.Windows.Forms.ToolBar toolBar1;
		private System.Windows.Forms.ToolBarButton butConnect;

		private Socket	m_clientSocket = null;
		IAsyncResult m_result;
		public AsyncCallback m_pfnCallBack = null;
		Process m_debuggedProcess = null;
		ArrayList m_nodeInfos = new ArrayList();
		private System.Windows.Forms.TextBox tbConsole;

		private SocketPacket		m_pendingPacket = new SocketPacket();
		private int					m_pendingBytes = 0;
		const ulong BUFFER_SIZE		= 512;
		const int MAX_STACK_DEPTH	= 10;
		private System.Windows.Forms.ToolTip toolTip1;
        private System.Windows.Forms.TextBox tbStats;
		private DelegateAddMemInfo	    m_delegateAddMemInfo;
        private TabPage tabUsageGraph;
        private CustomUIControls.Graphing.C2DPushGraph usageGraph;
        private TabPage tabSource;
        private RichTextBox richTextBox1;
        private Label labelFile;
        private DelegateRemoveMemInfo   m_delegateRemoveMemInfo;
        private MenuStrip menuStrip1;
        private ToolStripMenuItem toolStripMenuItem1;
        private ToolStripMenuItem saveFrameDataToolStripMenuItem;
        private SaveFileDialog saveFileDialog1;
        private ToolStripMenuItem loadFrameDataToolStripMenuItem;
        private OpenFileDialog openFileDialog1;
        private DelegateUpdateStats     m_delegateUpdateStats;
        private DelegateAssertion m_delegateAssertion;
        private TabControl tabControl2;
        private TabPage tabPage1;
        private TreeView treeView1;
        private TabPage tabPage2;
        private TreeView tvFrame;
        private Timer timer1;
        private int m_frameIndex = 0;
        private Label lFrameMemInfo;
        private ToolBarButton butPrevFrame;
        private ToolBarButton butNextFrame;
        private ToolBarButton butPlay;
        private ToolBarButton toolBarButton1;
        private ToolBarButton butFFwd;
        private ToolStripMenuItem exitToolStripMenuItem;
        private bool m_playing = false;

		class MemTreeNode : TreeNode
		{
			public ulong	    callerAddress = 0;
			public String	    fileName;
			public uint		    fileLine;
            public String       functionName;
			public MemInfo	    info = null;
            public int          numBytes = 0;
            public int          numBlocks = 0;
            public ArrayList    memBlocks = new ArrayList();

            public void UpdateNodeText()
            {
                float usagePercentage = 100.0f;
                MemTreeNode memParent = Parent as MemTreeNode;
                if (memParent != null)
                    usagePercentage = (float)numBytes * 100.0f / (float)memParent.numBytes;

                this.Text = functionName + "(" + numBytes + "b, " + numBlocks + " block(s), " + usagePercentage + "%)";
            }

		}

		public Form1()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();
			m_pendingPacket.dataBuffer = new byte[BUFFER_SIZE];
			m_delegateAddMemInfo = new DelegateAddMemInfo(this.AddNodeEntry);
            m_delegateRemoveMemInfo = new DelegateRemoveMemInfo(this.RemoveMemEntry);
            m_delegateUpdateStats = new DelegateUpdateStats(this.UpdateFrameStats);
            m_delegateAssertion = new DelegateAssertion(this.HandleAssertion);

            usageGraph.AddLine(0, Color.Green);
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
            this.components = new System.ComponentModel.Container();
            this.tabControl1 = new System.Windows.Forms.TabControl();
            this.tabUsageGraph = new System.Windows.Forms.TabPage();
            this.usageGraph = new CustomUIControls.Graphing.C2DPushGraph();
            this.tabSource = new System.Windows.Forms.TabPage();
            this.labelFile = new System.Windows.Forms.Label();
            this.richTextBox1 = new System.Windows.Forms.RichTextBox();
            this.toolBar1 = new System.Windows.Forms.ToolBar();
            this.butConnect = new System.Windows.Forms.ToolBarButton();
            this.toolBarButton1 = new System.Windows.Forms.ToolBarButton();
            this.butPrevFrame = new System.Windows.Forms.ToolBarButton();
            this.butNextFrame = new System.Windows.Forms.ToolBarButton();
            this.butPlay = new System.Windows.Forms.ToolBarButton();
            this.butFFwd = new System.Windows.Forms.ToolBarButton();
            this.tbConsole = new System.Windows.Forms.TextBox();
            this.toolTip1 = new System.Windows.Forms.ToolTip(this.components);
            this.tbStats = new System.Windows.Forms.TextBox();
            this.menuStrip1 = new System.Windows.Forms.MenuStrip();
            this.toolStripMenuItem1 = new System.Windows.Forms.ToolStripMenuItem();
            this.saveFrameDataToolStripMenuItem = new System.Windows.Forms.ToolStripMenuItem();
            this.loadFrameDataToolStripMenuItem = new System.Windows.Forms.ToolStripMenuItem();
            this.saveFileDialog1 = new System.Windows.Forms.SaveFileDialog();
            this.openFileDialog1 = new System.Windows.Forms.OpenFileDialog();
            this.tabControl2 = new System.Windows.Forms.TabControl();
            this.tabPage1 = new System.Windows.Forms.TabPage();
            this.treeView1 = new System.Windows.Forms.TreeView();
            this.tabPage2 = new System.Windows.Forms.TabPage();
            this.tvFrame = new System.Windows.Forms.TreeView();
            this.timer1 = new System.Windows.Forms.Timer(this.components);
            this.lFrameMemInfo = new System.Windows.Forms.Label();
            this.exitToolStripMenuItem = new System.Windows.Forms.ToolStripMenuItem();
            this.tabControl1.SuspendLayout();
            this.tabUsageGraph.SuspendLayout();
            this.tabSource.SuspendLayout();
            this.menuStrip1.SuspendLayout();
            this.tabControl2.SuspendLayout();
            this.tabPage1.SuspendLayout();
            this.tabPage2.SuspendLayout();
            this.SuspendLayout();
            // 
            // tabControl1
            // 
            this.tabControl1.Controls.Add(this.tabUsageGraph);
            this.tabControl1.Controls.Add(this.tabSource);
            this.tabControl1.Location = new System.Drawing.Point(275, 88);
            this.tabControl1.Name = "tabControl1";
            this.tabControl1.SelectedIndex = 0;
            this.tabControl1.Size = new System.Drawing.Size(544, 152);
            this.tabControl1.TabIndex = 1;
            // 
            // tabUsageGraph
            // 
            this.tabUsageGraph.Controls.Add(this.usageGraph);
            this.tabUsageGraph.Location = new System.Drawing.Point(4, 22);
            this.tabUsageGraph.Name = "tabUsageGraph";
            this.tabUsageGraph.Size = new System.Drawing.Size(536, 126);
            this.tabUsageGraph.TabIndex = 0;
            this.tabUsageGraph.Text = "Usage Graph";
            this.tabUsageGraph.UseVisualStyleBackColor = true;
            // 
            // usageGraph
            // 
            this.usageGraph.AutoAdjustPeek = false;
            this.usageGraph.BackColor = System.Drawing.Color.Black;
            this.usageGraph.GridColor = System.Drawing.Color.Green;
            this.usageGraph.GridSize = ((ushort)(15));
            this.usageGraph.HighQuality = true;
            this.usageGraph.LineInterval = ((ushort)(5));
            this.usageGraph.Location = new System.Drawing.Point(3, 5);
            this.usageGraph.MaxLabel = "500";
            this.usageGraph.MaxPeekMagnitude = 500;
            this.usageGraph.MinLabel = "0";
            this.usageGraph.MinPeekMagnitude = 0;
            this.usageGraph.Name = "usageGraph";
            this.usageGraph.ShowGrid = true;
            this.usageGraph.ShowLabels = true;
            this.usageGraph.Size = new System.Drawing.Size(525, 120);
            this.usageGraph.TabIndex = 0;
            this.usageGraph.TextColor = System.Drawing.Color.Yellow;
            // 
            // tabSource
            // 
            this.tabSource.Controls.Add(this.labelFile);
            this.tabSource.Controls.Add(this.richTextBox1);
            this.tabSource.Location = new System.Drawing.Point(4, 22);
            this.tabSource.Name = "tabSource";
            this.tabSource.Size = new System.Drawing.Size(536, 126);
            this.tabSource.TabIndex = 1;
            this.tabSource.Text = "Source";
            this.tabSource.UseVisualStyleBackColor = true;
            // 
            // labelFile
            // 
            this.labelFile.AutoSize = true;
            this.labelFile.Location = new System.Drawing.Point(14, 3);
            this.labelFile.Name = "labelFile";
            this.labelFile.Size = new System.Drawing.Size(0, 13);
            this.labelFile.TabIndex = 8;
            // 
            // richTextBox1
            // 
            this.richTextBox1.Location = new System.Drawing.Point(0, 19);
            this.richTextBox1.Name = "richTextBox1";
            this.richTextBox1.ShowSelectionMargin = true;
            this.richTextBox1.Size = new System.Drawing.Size(533, 104);
            this.richTextBox1.TabIndex = 3;
            this.richTextBox1.Text = "";
            this.richTextBox1.WordWrap = false;
            // 
            // toolBar1
            // 
            this.toolBar1.Buttons.AddRange(new System.Windows.Forms.ToolBarButton[] {
            this.butConnect,
            this.toolBarButton1,
            this.butPrevFrame,
            this.butNextFrame,
            this.butPlay,
            this.butFFwd});
            this.toolBar1.DropDownArrows = true;
            this.toolBar1.Location = new System.Drawing.Point(0, 24);
            this.toolBar1.Name = "toolBar1";
            this.toolBar1.ShowToolTips = true;
            this.toolBar1.Size = new System.Drawing.Size(819, 42);
            this.toolBar1.TabIndex = 4;
            this.toolBar1.ButtonClick += new System.Windows.Forms.ToolBarButtonClickEventHandler(this.toolBar1_ButtonClick);
            // 
            // butConnect
            // 
            this.butConnect.Name = "butConnect";
            this.butConnect.Text = "Connect";
            this.butConnect.ToolTipText = "Connect";
            // 
            // toolBarButton1
            // 
            this.toolBarButton1.Name = "toolBarButton1";
            this.toolBarButton1.Style = System.Windows.Forms.ToolBarButtonStyle.Separator;
            // 
            // butPrevFrame
            // 
            this.butPrevFrame.Enabled = false;
            this.butPrevFrame.Name = "butPrevFrame";
            this.butPrevFrame.Text = "<";
            this.butPrevFrame.ToolTipText = "Prev frame";
            // 
            // butNextFrame
            // 
            this.butNextFrame.Enabled = false;
            this.butNextFrame.Name = "butNextFrame";
            this.butNextFrame.Text = ">";
            this.butNextFrame.ToolTipText = "Next frame";
            // 
            // butPlay
            // 
            this.butPlay.Enabled = false;
            this.butPlay.Name = "butPlay";
            this.butPlay.Style = System.Windows.Forms.ToolBarButtonStyle.ToggleButton;
            this.butPlay.Text = "Play";
            // 
            // butFFwd
            // 
            this.butFFwd.Enabled = false;
            this.butFFwd.Name = "butFFwd";
            this.butFFwd.Text = ">>";
            // 
            // tbConsole
            // 
            this.tbConsole.BackColor = System.Drawing.SystemColors.InactiveBorder;
            this.tbConsole.Enabled = false;
            this.tbConsole.Location = new System.Drawing.Point(0, 246);
            this.tbConsole.Multiline = true;
            this.tbConsole.Name = "tbConsole";
            this.tbConsole.ScrollBars = System.Windows.Forms.ScrollBars.Vertical;
            this.tbConsole.Size = new System.Drawing.Size(815, 97);
            this.tbConsole.TabIndex = 5;
            // 
            // tbStats
            // 
            this.tbStats.Location = new System.Drawing.Point(0, 64);
            this.tbStats.Name = "tbStats";
            this.tbStats.ReadOnly = true;
            this.tbStats.Size = new System.Drawing.Size(272, 20);
            this.tbStats.TabIndex = 6;
            // 
            // menuStrip1
            // 
            this.menuStrip1.Items.AddRange(new System.Windows.Forms.ToolStripItem[] {
            this.toolStripMenuItem1});
            this.menuStrip1.Location = new System.Drawing.Point(0, 0);
            this.menuStrip1.Name = "menuStrip1";
            this.menuStrip1.Size = new System.Drawing.Size(819, 24);
            this.menuStrip1.TabIndex = 7;
            this.menuStrip1.Text = "menuStrip1";
            // 
            // toolStripMenuItem1
            // 
            this.toolStripMenuItem1.DropDownItems.AddRange(new System.Windows.Forms.ToolStripItem[] {
            this.saveFrameDataToolStripMenuItem,
            this.loadFrameDataToolStripMenuItem,
            this.exitToolStripMenuItem});
            this.toolStripMenuItem1.Name = "toolStripMenuItem1";
            this.toolStripMenuItem1.Size = new System.Drawing.Size(35, 20);
            this.toolStripMenuItem1.Text = "File";
            // 
            // saveFrameDataToolStripMenuItem
            // 
            this.saveFrameDataToolStripMenuItem.Enabled = false;
            this.saveFrameDataToolStripMenuItem.Name = "saveFrameDataToolStripMenuItem";
            this.saveFrameDataToolStripMenuItem.Size = new System.Drawing.Size(168, 22);
            this.saveFrameDataToolStripMenuItem.Text = "Save Frame Data";
            this.saveFrameDataToolStripMenuItem.Click += new System.EventHandler(this.saveFrameDataToolStripMenuItem_Click);
            // 
            // loadFrameDataToolStripMenuItem
            // 
            this.loadFrameDataToolStripMenuItem.Name = "loadFrameDataToolStripMenuItem";
            this.loadFrameDataToolStripMenuItem.Size = new System.Drawing.Size(168, 22);
            this.loadFrameDataToolStripMenuItem.Text = "Load Frame Data";
            this.loadFrameDataToolStripMenuItem.Click += new System.EventHandler(this.loadFrameDataToolStripMenuItem_Click);
            // 
            // saveFileDialog1
            // 
            this.saveFileDialog1.DefaultExt = "MemInfo";
            this.saveFileDialog1.Filter = "Frame data|*.MemInfo";
            // 
            // openFileDialog1
            // 
            this.openFileDialog1.DefaultExt = "MemInfo";
            this.openFileDialog1.FileName = "openFileDialog1";
            this.openFileDialog1.Filter = "Frame data|*.MemInfo";
            // 
            // tabControl2
            // 
            this.tabControl2.Controls.Add(this.tabPage1);
            this.tabControl2.Controls.Add(this.tabPage2);
            this.tabControl2.Location = new System.Drawing.Point(0, 88);
            this.tabControl2.Name = "tabControl2";
            this.tabControl2.SelectedIndex = 0;
            this.tabControl2.Size = new System.Drawing.Size(269, 152);
            this.tabControl2.TabIndex = 10;
            // 
            // tabPage1
            // 
            this.tabPage1.Controls.Add(this.treeView1);
            this.tabPage1.Location = new System.Drawing.Point(4, 22);
            this.tabPage1.Name = "tabPage1";
            this.tabPage1.Padding = new System.Windows.Forms.Padding(3);
            this.tabPage1.Size = new System.Drawing.Size(261, 126);
            this.tabPage1.TabIndex = 0;
            this.tabPage1.Text = "Global";
            this.tabPage1.UseVisualStyleBackColor = true;
            // 
            // treeView1
            // 
            this.treeView1.Location = new System.Drawing.Point(0, 0);
            this.treeView1.Name = "treeView1";
            this.treeView1.Size = new System.Drawing.Size(260, 126);
            this.treeView1.TabIndex = 4;
            this.treeView1.DoubleClick += new System.EventHandler(this.DoubleClickNode);
            this.treeView1.MouseMove += new System.Windows.Forms.MouseEventHandler(this.treeView1_MouseMove);
            // 
            // tabPage2
            // 
            this.tabPage2.Controls.Add(this.tvFrame);
            this.tabPage2.Location = new System.Drawing.Point(4, 22);
            this.tabPage2.Name = "tabPage2";
            this.tabPage2.Padding = new System.Windows.Forms.Padding(3);
            this.tabPage2.Size = new System.Drawing.Size(261, 126);
            this.tabPage2.TabIndex = 1;
            this.tabPage2.Text = "Frame";
            this.tabPage2.UseVisualStyleBackColor = true;
            // 
            // tvFrame
            // 
            this.tvFrame.Location = new System.Drawing.Point(0, 0);
            this.tvFrame.Name = "tvFrame";
            this.tvFrame.Size = new System.Drawing.Size(260, 126);
            this.tvFrame.TabIndex = 0;
            this.tvFrame.DoubleClick += new System.EventHandler(this.DoubleClickNode);
            this.tvFrame.MouseMove += new System.Windows.Forms.MouseEventHandler(this.tvFrame_MouseMove);
            // 
            // timer1
            // 
            this.timer1.Tick += new System.EventHandler(this.timer1_Tick);
            // 
            // lFrameMemInfo
            // 
            this.lFrameMemInfo.AutoSize = true;
            this.lFrameMemInfo.Location = new System.Drawing.Point(281, 66);
            this.lFrameMemInfo.Name = "lFrameMemInfo";
            this.lFrameMemInfo.Size = new System.Drawing.Size(0, 13);
            this.lFrameMemInfo.TabIndex = 13;
            // 
            // exitToolStripMenuItem
            // 
            this.exitToolStripMenuItem.Name = "exitToolStripMenuItem";
            this.exitToolStripMenuItem.Size = new System.Drawing.Size(168, 22);
            this.exitToolStripMenuItem.Text = "Exit";
            this.exitToolStripMenuItem.Click += new System.EventHandler(this.exitToolStripMenuItem_Click);
            // 
            // Form1
            // 
            this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
            this.ClientSize = new System.Drawing.Size(819, 348);
            this.Controls.Add(this.lFrameMemInfo);
            this.Controls.Add(this.tabControl2);
            this.Controls.Add(this.tbStats);
            this.Controls.Add(this.tbConsole);
            this.Controls.Add(this.toolBar1);
            this.Controls.Add(this.tabControl1);
            this.Controls.Add(this.menuStrip1);
            this.MainMenuStrip = this.menuStrip1;
            this.Name = "Form1";
            this.Text = "Mem Info";
            this.Closing += new System.ComponentModel.CancelEventHandler(this.Form1_Closing);
            this.tabControl1.ResumeLayout(false);
            this.tabUsageGraph.ResumeLayout(false);
            this.tabSource.ResumeLayout(false);
            this.tabSource.PerformLayout();
            this.menuStrip1.ResumeLayout(false);
            this.menuStrip1.PerformLayout();
            this.tabControl2.ResumeLayout(false);
            this.tabPage1.ResumeLayout(false);
            this.tabPage2.ResumeLayout(false);
            this.ResumeLayout(false);
            this.PerformLayout();

		}
		#endregion

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main() 
		{
			Application.Run(new Form1());
		}

		private void ScrollToLine(uint line)
		{
			//line = 1;
			string selectedLine = this.richTextBox1.Lines[line - 1];
			int selStart = 0;
			for (int i = 0; i < line - 1; ++i)
			{
				// +1 == newline
				string thisLine = richTextBox1.Lines[i];
				selStart += richTextBox1.Lines[i].Length + 1;
			}

			richTextBox1.Focus();
			richTextBox1.SelectionLength = selectedLine.Length;
			richTextBox1.SelectionStart = selStart;
			richTextBox1.ScrollToCaret();
		}

		private void DoubleClickNode(object sender, System.EventArgs e)
		{
			TreeView tree = sender as TreeView;
			if (tree != null && tree.SelectedNode != null)
			{
				MemTreeNode memNode = (MemTreeNode)tree.SelectedNode;
				try
				{
					this.richTextBox1.LoadFile(memNode.fileName, RichTextBoxStreamType.PlainText);
					ScrollToLine(memNode.fileLine);
                    labelFile.Text = memNode.fileName + "@" + memNode.fileLine;
				}
				catch (System.IO.IOException)
				{
                    labelFile.Text = "";
				}
			}
		}

		[StructLayout(LayoutKind.Explicit)]
		struct SocketPacket
		{
			public enum CommandID
			{
				PROCESS_ID = 1,
				MODULE_INFO,
				ALLOC,
				FREE,
                FRAME_START,
                FRAME_END,
                TAG_BLOCK,
                ASSERT,
				MAX
			};

			[FieldOffset(0)]
			public byte[] dataBuffer;
			[FieldOffset(0)]
			public uint[] intBuffer;
		}

		public void WaitForData()
		{
			try
			{
                if (m_clientSocket == null)
                    return;

				if  ( m_pfnCallBack == null ) 
				{
					m_pfnCallBack = new AsyncCallback (OnDataReceived);
				}
				SocketPacket theSocPkt = new SocketPacket ();
				theSocPkt.dataBuffer = new byte[BUFFER_SIZE];
				// Start listening to the data asynchronously
				m_result = m_clientSocket.BeginReceive (theSocPkt.dataBuffer,
					0, theSocPkt.dataBuffer.Length,
					SocketFlags.None, 
					m_pfnCallBack, 
					theSocPkt);
			}
			catch(SocketException se)
			{
				MessageBox.Show (se.Message );
			}

		}

        private static uint AsUint(SocketPacket packet, int i)
        {
            return packet.intBuffer[i >> 2];
        }

		private MemTreeNode FindNode(ulong address, TreeNodeCollection nodes, bool recursive)
		{
			foreach (TreeNode node in nodes)
			{
				MemTreeNode memNode = (MemTreeNode)node;
				if (memNode.callerAddress == address)
					return memNode;

				if (recursive)
				{
					memNode = FindNode(address, node.Nodes, true);
                    if (memNode != null)
                        return memNode;
				}
			}
			return null;
		}
        private static void InitMemNodeFromSymbol(MemTreeNode node, StackTracer.Symbol symbol)
        {
            node.functionName = symbol.functionName;
            node.fileName = symbol.fileName;
            node.fileLine = symbol.line;
        }
        private static void PropagateInfoUpwards(MemTreeNode node, MemInfo info)
        {
            if (node.Parent != null)
                PropagateInfoUpwards(node.Parent as MemTreeNode, info);

            ++node.numBlocks;
            node.numBytes += (int)info.m_bytes;
            node.UpdateNodeText();
        }

        private void AddPerFrameNodeEntry(MemInfo info)
        {
            TreeNodeCollection nodes = tvFrame.Nodes;
            for (int i = 0; i < info.m_depth; ++i)
            {
                if (info.m_callStack[i] == 0)
                    continue;

                StackTracer.Symbol symbol = StackTracer.GetSymbolForAddress(info.m_callStack[i]);
                if (symbol.functionName == null)
                    continue;

                MemTreeNode node = new MemTreeNode();
                InitMemNodeFromSymbol(node, symbol);
                node.Text = symbol.functionName;
                node.Tag = "" + info.m_bytes + " byte(s)";
                nodes.Add(node);
                nodes = nodes[nodes.Count - 1].Nodes;
            }
        }

        private static int FindFirstValidSymbolIndex(MemInfo info)
        {
            for (int i = 0; i < info.m_depth; ++i)
            {
                if (info.m_callStack[i] == 0)
                    continue;

                StackTracer.Symbol symbol = StackTracer.GetSymbolForAddress(info.m_callStack[i]);
                if (symbol.functionName != null)
                    return i;
            }
            return -1;
        }

        private MemTreeNode AddMemTreeNode(uint address, StackTracer.Symbol symbol,
            TreeNodeCollection nodes)
        {
            MemTreeNode node = new MemTreeNode();
            node.callerAddress = address;
            InitMemNodeFromSymbol(node, symbol);
            nodes.Add(node);
            return node;
        }
        private void AddNodeEntry(MemInfo info)
		{
            int rootIndex = FindFirstValidSymbolIndex(info);
            if (rootIndex < 0)
                return;

            StackTracer.Symbol symbol = StackTracer.GetSymbolForAddress(info.m_callStack[rootIndex]);
            MemTreeNode rootNode = FindNode(info.m_callStack[rootIndex], treeView1.Nodes, true);
            if (rootNode == null)
            {
                rootNode = AddMemTreeNode(info.m_callStack[rootIndex], symbol, treeView1.Nodes);
            }
            TreeNodeCollection parentNodes = rootNode.Nodes;
            for (int i = rootIndex + 1; i < info.m_depth; ++i)
            {
                uint callerAddress = info.m_callStack[i];
                MemTreeNode thisNode = FindNode(callerAddress, parentNodes, false);
                if (thisNode == null)
                {
                    symbol = StackTracer.GetSymbolForAddress(callerAddress);
                    thisNode = AddMemTreeNode(callerAddress, symbol, parentNodes);
                }
                if (i == info.m_depth - 1)
                {
                    PropagateInfoUpwards(thisNode, info);
                    thisNode.memBlocks.Add(info.m_address);
                }
                parentNodes = thisNode.Nodes;
            }
            UpdateNodeTexts(treeView1.Nodes);
            AddPerFrameNodeEntry(info);

            if (m_clientSocket != null)
            {
                FrameInfo.GetCurrentFrameInfo().AddAllocation(info);
                saveFrameDataToolStripMenuItem.Enabled = true;
            }
		}

        private void RemoveNodeHierarchyUpwards(TreeNode node)
        {
            MemTreeNode memNode = node.Parent as MemTreeNode;
            node.Parent.Nodes.Remove(node);
            if (memNode != null && memNode.Nodes.Count == 0 && memNode.memBlocks.Count == 0)
                RemoveNodeHierarchyUpwards(memNode);
        }

        private void PropagateMemFreeUpwards(TreeNode node, MemInfo info)
        {
            if (node.Parent != null)
                PropagateMemFreeUpwards(node.Parent, info);

            MemTreeNode memNode = node as MemTreeNode;
            memNode.numBytes -= (int)info.m_bytes;
            --memNode.numBlocks;
            memNode.UpdateNodeText();
        }

        private static MemTreeNode FindNodeWithMemBlock(uint memAddress, TreeNodeCollection nodes)
        {
            foreach (TreeNode node in nodes)
            {
                MemTreeNode memNode = node as MemTreeNode;
                if (memNode.memBlocks.IndexOf(memAddress) >= 0)
                    return memNode;

                memNode = FindNodeWithMemBlock(memAddress, node.Nodes);
                if (memNode != null)
                    return memNode;
            }
            return null;
        }

        private void RemoveMemEntry(MemInfo info, TreeNodeCollection nodes)
        {
            foreach (TreeNode node in nodes)
            {
                MemTreeNode memNode = node as MemTreeNode;
                if (memNode.memBlocks.IndexOf(info.m_address) >= 0)
                {
                    memNode.memBlocks.Remove(info.m_address);
                    PropagateMemFreeUpwards(node, info);
                    if (memNode.memBlocks.Count == 0)
                    {
                        RemoveNodeHierarchyUpwards(node);
                        return;
                    }

                }
                RemoveMemEntry(info, node.Nodes);
            }
        }

        private void UpdateNodeTexts(TreeNodeCollection nodes)
        {
            foreach (TreeNode node in nodes)
            {
                MemTreeNode memNode = node as MemTreeNode;
                memNode.UpdateNodeText();
                UpdateNodeTexts(node.Nodes);
            }
        }

        private void RemoveMemEntry(uint memAddress)
        {
            MemInfo info = MemInfo.InfoForAddress(memAddress);
            MemInfo.Remove(memAddress);
            TreeNodeCollection nodes = treeView1.Nodes;
            RemoveMemEntry(info, nodes);
            UpdateNodeTexts(treeView1.Nodes);
            if (m_clientSocket != null)
                FrameInfo.GetCurrentFrameInfo().AddFree(info);
        }

        private void UpdateUsageGraph(int allocatedKilobytes)
        {
            int pushVal = allocatedKilobytes;
            if (pushVal > usageGraph.MaxPeekMagnitude)
            {
                usageGraph.MaxPeekMagnitude = pushVal + 10;
                usageGraph.MaxLabel = usageGraph.MaxPeekMagnitude.ToString();
            }
            usageGraph.Push(pushVal, 0);
            //usageGraph.Refresh();
        }

        private void UpdateFrameStats()
        {
            tbStats.Text = " " + MemInfo.ms_allocatedBytes + " byte(s) in " + MemInfo.ms_allocatedBlocks + " block(s)";
            UpdateUsageGraph((int)MemInfo.ms_allocatedBytes / 1000);
        }
        private void TagMemoryBlock(uint memAddress, String tag)
        {
            MemTreeNode node = FindNodeWithMemBlock(memAddress, treeView1.Nodes);
            if (node != null)
                node.Tag = tag;
        }
        private void HandleAssertion(String expr, uint[] callStack)
        {
            tbConsole.Text += expr + "\r\n";
        }

        // @note OK, my C# knowledge sucks
        private static String StringFromNullTerminatedChars(char[] chars)
        {
            String str = new String(chars);
            int nullIndex = str.IndexOf("\0");
            return str.Remove(nullIndex, str.Length - nullIndex);
        }
        private static String StringFromPacket(SocketPacket packet, int dataIdx)
        {
            System.Text.Decoder d = System.Text.Encoding.UTF8.GetDecoder();
            int charLen = d.GetCharCount(packet.dataBuffer, dataIdx, 256);
            char[] nameChars = new char[charLen];
            d.GetChars(packet.dataBuffer, dataIdx, 256, nameChars, 0);
            return StringFromNullTerminatedChars(nameChars);
        }

        private bool HandlePendingData(int msgSize, int dataIdx, int iRx, SocketPacket packet)
        {
            if (dataIdx + msgSize > iRx)
			{
				m_pendingBytes = iRx - dataIdx;
				for (int i = 0; i < m_pendingBytes; ++i)
					m_pendingPacket.dataBuffer[i] = packet.dataBuffer[dataIdx + i];
                return true;
			}
            return false;
        }
		public  void OnDataReceived(IAsyncResult asyn)
		{
			if (m_clientSocket == null)
				return;

			try
			{
				SocketPacket recPacket = (SocketPacket)asyn.AsyncState ;
				int iRx = m_clientSocket.EndReceive (asyn);

				SocketPacket packet = recPacket;
				if (m_pendingBytes > 0)
				{
					packet = new SocketPacket();
					packet.dataBuffer = new byte[BUFFER_SIZE*2];					
					for (int i = 0; i < m_pendingBytes; ++i)
						packet.dataBuffer[i] = m_pendingPacket.dataBuffer[i];
					for (int i = 0; i < iRx; ++i)
						packet.dataBuffer[i + m_pendingBytes] = recPacket.dataBuffer[i];

					iRx += m_pendingBytes;
					m_pendingBytes = 0;
				}

				uint processId = 0;
				int dataIdx = 0;
				while (dataIdx < iRx)
				{
                    if (HandlePendingData(4, dataIdx, iRx, packet))
                        break;

					SocketPacket.CommandID commandId = (SocketPacket.CommandID)AsUint(packet, dataIdx);
					if (commandId == SocketPacket.CommandID.PROCESS_ID)
					{
						processId = AsUint(packet, dataIdx + 4);
						m_debuggedProcess = Process.GetProcessById((int)processId);
						dataIdx += 8;
					}
					else if (commandId == SocketPacket.CommandID.MODULE_INFO)
					{
						ulong modBase = AsUint(packet, dataIdx + 4);
                        String szData = StringFromPacket(packet, dataIdx + 8);
                        StackTracer.InitModuleInfo(szData, modBase, m_debuggedProcess.Handle);
						dataIdx += 8 + 256;
					}
					else if (commandId == SocketPacket.CommandID.ALLOC)
					{
						const int kMsgSize = 4 + 8 + 4*MAX_STACK_DEPTH;
                        if (HandlePendingData(kMsgSize, dataIdx, iRx, packet))
                            break;

						MemInfo blockInfo = new MemInfo();
						blockInfo.m_address = AsUint(packet, dataIdx+4);
						blockInfo.m_bytes	= AsUint(packet, dataIdx+8);
                        bool hadEntry = false;
						for (int i = 0; i < MAX_STACK_DEPTH; ++i)
						{
							blockInfo.m_callStack[i] = AsUint(packet, dataIdx + 12 + i * 4);
                            if (blockInfo.m_callStack[i] != 0)
                            {
                                StackTracer.GetSymbolForAddress(blockInfo.m_callStack[i]);
                                hadEntry = true;
                            }
                            if (hadEntry && blockInfo.m_callStack[i] != 0)
                                blockInfo.m_depth = i + 1;
						}
                        MemInfo.Add(blockInfo);
						Invoke(m_delegateAddMemInfo, new Object[] {blockInfo});

						dataIdx += kMsgSize;
					}
					else if (commandId == SocketPacket.CommandID.FREE)
					{
						const int kMsgSize = 4 + 8 + 4*MAX_STACK_DEPTH;
                        if (HandlePendingData(kMsgSize, dataIdx, iRx, packet))
                            break;

						uint addr = AsUint(packet, dataIdx + 4);
                        Invoke(m_delegateRemoveMemInfo, new Object[] { addr });
						dataIdx += kMsgSize;
					}
                    else if (commandId == SocketPacket.CommandID.FRAME_START)
                    {
                        FrameInfo.StartFrame();
                        dataIdx += 4;
                    }
                    else if (commandId == SocketPacket.CommandID.FRAME_END)
                    {
                        Invoke(m_delegateUpdateStats);
                        dataIdx += 4;
                    }
                    else if (commandId == SocketPacket.CommandID.TAG_BLOCK)
                    {
                        const int kMsgSize = 4 + 4 + 256;
                        if (HandlePendingData(kMsgSize, dataIdx, iRx, packet))
                            break;

                        uint addr = AsUint(packet, dataIdx + 4);
                        String tag = StringFromPacket(packet, dataIdx + 8);
                        TagMemoryBlock(addr, tag);

                        dataIdx += kMsgSize;
                    }
                    else if (commandId == SocketPacket.CommandID.ASSERT)
                    {
                        const int kMsgSize = 4 + 256 + MAX_STACK_DEPTH * 4;
                        if (HandlePendingData(kMsgSize, dataIdx, iRx, packet))
                            break;

                        String expr = StringFromPacket(packet, dataIdx + 4);
                        uint[] callStack = new uint[MAX_STACK_DEPTH];
                        // @todo: fill callstack
                        Invoke(m_delegateAssertion, new Object[] { expr, callStack });

                        dataIdx += kMsgSize;
                    }
				}

				WaitForData();
			}
			catch (ObjectDisposedException )
			{
				System.Diagnostics.Debugger.Log(0,"1","\nOnDataReceived: Socket has been closed\n");
			}
			catch(SocketException se)
			{
				MessageBox.Show (se.Message );
                CloseSocket();
			}
		}	

		private void CloseSocket()
		{
			if (m_clientSocket != null)
			{
				m_clientSocket.Shutdown(SocketShutdown.Both);
				m_clientSocket.Close ();
				m_clientSocket = null;
			}		
		}

		private void toolBar1_ButtonClick(object sender, System.Windows.Forms.ToolBarButtonClickEventArgs e)
		{
            if (e.Button == butNextFrame)
            {
                butNextFrame_Click(sender, e);
                return;
            }
            else if (e.Button == butPrevFrame)
            {
                butPrevFrame_Click(sender, e);
                return;
            }
            else if (e.Button == butPlay)
            {
                butPlay_Click(sender, e);
                return;
            }
            else if (e.Button == butFFwd)
            {
                butFFwd_Click(sender, e);
                return;
            }

			if (m_clientSocket != null)
			{
                CloseSocket();
				butConnect.Text = "Connect";
				tbConsole.Text += "Disconnected\r\n";
				return;
			}

			try
			{
                ConnectDialog cd = new ConnectDialog();
                if (cd.ShowDialog(this) != DialogResult.OK)
                    return;

				// Create the socket instance
				m_clientSocket = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
				
				// Cet the remote IP address
                String strIP = cd.GetServer();
				IPAddress ip = IPAddress.Parse(strIP);
				int iPortNo = 5002;
				IPEndPoint ipEnd = new IPEndPoint(ip, iPortNo);
				m_clientSocket.Connect ( ipEnd );

				tbConsole.Text += "Connected to " + ipEnd + "\r\n";

				if (m_clientSocket.Connected) 
				{
					butConnect.Text = "Disconnect";
					WaitForData();
				}
			}
			catch (SocketException se)
			{
				string str;
				str = "\nConnection failed, is the server running?\n" + se.Message;
				MessageBox.Show (str);
				m_clientSocket = null;
			}		
		}

		private void Form1_Closing(object sender, System.ComponentModel.CancelEventArgs e)
		{
			CloseSocket();
		}

		private void treeView1_MouseMove(object sender, System.Windows.Forms.MouseEventArgs e)
		{
            MemTreeNode currentNode = this.treeView1.GetNodeAt(e.X, e.Y) as MemTreeNode;
			if (null != currentNode && currentNode.Tag != null)
			{
                String newTooltip = (String)currentNode.Tag;// " " + currentNode.memBlocks.Count + " block(s)";
				//if (currentNode.Tag.ToString() != this.toolTip1.GetToolTip(this.treeView1).ToString( ))
                if (newTooltip != this.toolTip1.GetToolTip(this.treeView1).ToString())
				{
                    this.toolTip1.SetToolTip(this.treeView1, newTooltip);
				}
			}
			else
			{
				this.toolTip1.SetToolTip(this.treeView1,"");
			}
		}

        private void LoadFrameInfos(String fileName)
        {
            Stream s = File.Open(fileName, FileMode.Open);
            BinaryFormatter formatter = new BinaryFormatter();
            StackTracer.Load(s, formatter);
            FrameInfo.Load(s, formatter);
            s.Close();
        }
        private void SaveFrameInfos(String fileName)
        {
            Stream s = File.Open(fileName, FileMode.Create);
            BinaryFormatter formatter = new BinaryFormatter();
            StackTracer.Save(s, formatter);
            FrameInfo.Save(s, formatter);
            s.Close();
        }

        private void saveFrameDataToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (saveFileDialog1.ShowDialog() == DialogResult.OK)
            {
                SaveFrameInfos(saveFileDialog1.FileName);
            }
        }

        private void loadFrameDataToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (m_clientSocket != null)
            {
                if (MessageBox.Show(this, "Frame-by-frame analysis will disconnect. Are you sure?", "", MessageBoxButtons.YesNo) != DialogResult.Yes)
                    return;

                CloseSocket();
                butConnect.Text = "Connect";
            }

            if (openFileDialog1.ShowDialog() == DialogResult.OK)
            {
                LoadFrameInfos(openFileDialog1.FileName);
                butPrevFrame.Enabled = true;
                butNextFrame.Enabled = true;
                butPlay.Enabled = true;
                butFFwd.Enabled = true;
                m_frameIndex = 0;
                MemInfo.Clear();
                treeView1.Nodes.Clear();
                UpdateMemFrame();
            }
        }

        private void UpdateMemFrame()
        {
            tvFrame.Nodes.Clear();

            FrameInfo frame = FrameInfo.GetFrameInfo(m_frameIndex);
            uint frameBytes = 0;
            foreach (MemInfo info in frame.m_blocksAllocated)
            {
                MemInfo.Add(info);
                AddNodeEntry(info);
                frameBytes += info.m_bytes;
            }
            foreach (MemInfo info in frame.m_blocksFreed)
            {
                RemoveMemEntry(info.m_address);
            }
            lFrameMemInfo.Text = "Frame: " + m_frameIndex + "/" + (FrameInfo.GetNumFrameInfos()-1) +
                ", " + frameBytes + " bytes in " + frame.m_blocksAllocated.Count + " block(s) allocated, " +
                frame.m_blocksFreed.Count + " block(s) freed";
            UpdateFrameStats();
        }

        private void NextMemFrame()
        {
            ++m_frameIndex;
            if (m_frameIndex >= FrameInfo.GetNumFrameInfos() - 1)
            {
                m_frameIndex = FrameInfo.GetNumFrameInfos() - 1;
                butNextFrame.Enabled = false;
                butFFwd.Enabled = false;
            }
            else
            {
                butNextFrame.Enabled = true;
                butFFwd.Enabled = true;
            }
            UpdateMemFrame();
        }
        private void butNextFrame_Click(object sender, EventArgs e)
        {
            NextMemFrame();
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            NextMemFrame();
            if (!butNextFrame.Enabled)
                timer1.Enabled = false;
        }

        private void butPlay_Click(object sender, EventArgs e)
        {
            m_playing = !m_playing;
            butPlay.Text = m_playing ? "Pause" : "Play";
            timer1.Enabled = m_playing;
        }

        private void butFFwd_Click(object sender, EventArgs e)
        {
            int numFrameInfos = FrameInfo.GetNumFrameInfos();
            for (int i = m_frameIndex + 1; i < numFrameInfos; ++i)
            {
                FrameInfo frame = FrameInfo.GetFrameInfo(i);
                if (frame.m_blocksAllocated.Count != 0 || frame.m_blocksFreed.Count != 0)
                {
                    m_frameIndex = i;
                    UpdateMemFrame();
                    break;
                }
            }
        }

        private void butPrevFrame_Click(object sender, EventArgs e)
        {
            // @YUCK! Slow, but reliable.
            int newFrameIndex = m_frameIndex - 1;
            MemInfo.Clear();
            treeView1.Nodes.Clear();
            for (m_frameIndex = 0; m_frameIndex <= newFrameIndex; ++m_frameIndex)
                UpdateMemFrame();
            m_frameIndex = newFrameIndex;
            if (m_frameIndex < FrameInfo.GetNumFrameInfos() - 1)
            {
                butNextFrame.Enabled = true;
                butFFwd.Enabled = true;
            }
        }

        private void tvFrame_MouseMove(object sender, MouseEventArgs e)
        {
            MemTreeNode currentNode = this.tvFrame.GetNodeAt(e.X, e.Y) as MemTreeNode;
            if (null != currentNode && currentNode.Tag != null)
            {
                String newTooltip = (String)currentNode.Tag;
                if (newTooltip != this.toolTip1.GetToolTip(this.tvFrame).ToString())
                {
                    this.toolTip1.SetToolTip(this.tvFrame, newTooltip);
                }
            }
            else
            {
                this.toolTip1.SetToolTip(this.tvFrame, "");
            }
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Close();
        }
    }
}
