#define USE_DIA     

using System;
using System.Diagnostics;
using System.Runtime.InteropServices;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters.Binary;
using System.Collections;
using System.IO;
using Dia2Lib;

namespace WindowsApplication1
{
	/// <summary>
	/// Summary description for StackTracer.
	/// </summary>
	public class StackTracer
	{
		public StackTracer()
		{
		}

		public struct Symbol
		{
			public string	functionName;
			public string	fileName;
			public uint		line;
			public uint		address;
			public ulong	functionAddress;
		};

        static String m_pdbName;
        static ulong m_moduleBase = 0;
#if USE_DIA
        static IDiaDataSource   m_diaSource;
        static IDiaSession      m_session = null;
        static Hashtable        m_symbols = new Hashtable();
#else
        static uint m_moduleSize;
        static IntPtr m_hProcess;
#endif
		public static bool InitModuleInfo(String pdbName, ulong moduleBase, IntPtr hDebuggedProcess)
		{
#if USE_DIA
            if (m_pdbName != pdbName || moduleBase != m_moduleBase)
            {
                if (m_moduleBase != 0)
                    Close();

                m_pdbName = pdbName;
                m_moduleBase = moduleBase;

                m_diaSource = new DiaSourceClass();
                try
                {
                    m_diaSource.loadDataFromPdb(pdbName);
                    m_diaSource.openSession(out m_session);
                    m_session.loadAddress = moduleBase;
                }
                catch (Exception)
                {
                    m_diaSource = null;
                }
                return true;
            }
            return true;
#else
            if (m_pdbName != pdbName || moduleBase != m_moduleBase || hDebuggedProcess != m_hProcess)
            {
                if (m_moduleBase != 0)
                    Close();

                m_pdbName = pdbName;
                m_moduleBase = moduleBase;
                FileStream f = new FileStream(pdbName, FileMode.Open, FileAccess.Read);
                m_moduleSize = (uint)f.Length;
                f = null;

                m_hProcess = hDebuggedProcess;
                DbgHelp.SymSetOptions(DbgHelp.SymOpt.UNDNAME | DbgHelp.SymOpt.DEFERRED_LOADS | DbgHelp.SymOpt.LOAD_LINES);
                if (DbgHelp.SymInitialize(m_hProcess, null, false))
                {
                    if (DbgHelp.SymLoadModule64(m_hProcess, IntPtr.Zero,
                        m_pdbName, null, m_moduleBase, m_moduleSize) != 0)
                    {
                        return true;
                    }
                }
                return false;
            }
            return true;
#endif
		}

		public static void Close()
		{
#if USE_DIA
            m_session = null;
            m_diaSource = null;
#else
            DbgHelp.SymCleanup(m_hProcess);
#endif
		}

		public static Symbol GetSymbolForAddress(uint addr)
		{
#if USE_DIA
            Symbol ret;
            if (addr == 0)
                return new Symbol();
            if (m_symbols.ContainsKey(addr))
            {
                return (Symbol)m_symbols[addr];
            }
            
            ret = new Symbol();
            IDiaSymbol symbol = null;
            SymTagEnum tagEnum = SymTagEnum.SymTagFunction;
            m_session.findSymbolByVA((ulong)addr, tagEnum, out symbol);
            if (symbol == null)
            {
                tagEnum = SymTagEnum.SymTagNull;
                m_session.findSymbolByVA((ulong)addr, tagEnum, out symbol);
            }

            if (symbol != null)
            {
                ret.functionName = (tagEnum == SymTagEnum.SymTagNull ? symbol.undecoratedName : symbol.name);
                ret.address = addr;
                ret.functionAddress = symbol.virtualAddress;

                IDiaEnumLineNumbers lineNumbers;
                //m_session.findLinesByAddr(symbol.addressSection, symbol.addressOffset, 1, out lineNumbers);
                m_session.findLinesByVA(addr, 1, out lineNumbers);
                uint celt = 0;
                IDiaLineNumber line;
                while (true)
                {
                    lineNumbers.Next(1, out line, out celt);
                    if (celt == 1)
                    {
                        ret.fileName = line.sourceFile.fileName;
                        ret.line = line.lineNumber;
                        break;
                    }
                    if (celt != 1)
                        break;
                }
            }
            m_symbols.Add(addr, ret);
#else
			Symbol ret = new Symbol();
			ulong disp = 0;
			DbgHelp.SYMBOL_INFO symbolInfo = new DbgHelp.SYMBOL_INFO();
            symbolInfo.SizeOfStruct = (uint)Marshal.SizeOf(symbolInfo);
			symbolInfo.MaxNameLen = 1024;
			if (DbgHelp.SymFromAddr(m_hProcess, addr, ref disp, ref symbolInfo))
			{
				uint dispu = 0;
				DbgHelp.IMAGEHLP_LINE64 lineInfo = new DbgHelp.IMAGEHLP_LINE64();
				lineInfo.SizeOfStruct = (uint)Marshal.SizeOf(lineInfo);
				DbgHelp.SymGetLineFromAddr64(m_hProcess, addr, ref dispu, ref lineInfo);
				ret.fileName = Marshal.PtrToStringAnsi(lineInfo.FileName);
				ret.line = lineInfo.LineNumber;
				ret.functionName = symbolInfo.Name;
				ret.functionAddress = symbolInfo.Address;
				ret.address = addr;
			}
#endif
            return ret;
		}

        static public void Save(Stream s, BinaryFormatter formatter)
        {
            formatter.Serialize(s, m_pdbName);
            formatter.Serialize(s, m_moduleBase);
#if !USE_DIA
            formatter.Serialize(s, m_moduleSize);
            formatter.Serialize(s, m_hProcess);
#endif
        }
        static public void Load(Stream s, BinaryFormatter formatter)
        {
            String pdbName = (String)formatter.Deserialize(s);
            ulong moduleBase = (ulong)formatter.Deserialize(s);
#if USE_DIA
            IntPtr hProcess = new IntPtr();
            InitModuleInfo(pdbName, moduleBase, hProcess);
#else
            uint moduleSize = (uint)formatter.Deserialize(s);
            IntPtr hProcess = (IntPtr)formatter.Deserialize(s);
            InitModuleInfo(pdbName, moduleBase, hProcess);
#endif
        }
	}
}
