#include "MemManager.h"

namespace
{
	struct MemManagerInitializer
	{
		static void __cdecl ExitMem()
		{
			rdecore::MemManager::Exit();
		}
		MemManagerInitializer()
		{
			::atexit(&ExitMem);
		}
	} s_memManagerInitializer;

	rdecore::MemManager::OnAlloc	onAllocHandler(0);
	rdecore::MemManager::OnFree		onFreeHandler(0);
}

//-----------------------------------------------------------------------------
void* __cdecl operator new(size_t bytes) 
{
	return rdecore::MemManager::Alloc(bytes);
}
void* __cdecl operator new[](size_t bytes) 
{
	return rdecore::MemManager::Alloc(bytes);
}
void* __cdecl operator new(size_t bytes, const std::nothrow_t&) throw() 
{
	return rdecore::MemManager::Alloc(bytes);
} 
void* __cdecl operator new[](size_t bytes, const std::nothrow_t&) throw() 
{
	return rdecore::MemManager::Alloc(bytes);
} 

//-----------------------------------------------------------------------------
void __cdecl operator delete(void* ptr) throw()
{
	rdecore::MemManager::Free(ptr);
}
void __cdecl operator delete[](void* ptr) throw()
{
	rdecore::MemManager::Free(ptr);
}

namespace rdecore
{
#ifdef RDE_CORE_MEMMGR_HEADER
#	include RDE_CORE_MEMMGR_HEADER
#else

static int s_numAllocations(0);
//-----------------------------------------------------------------------------
void* MemManager::Alloc(size_t bytes, const char* tag)
{
	++s_numAllocations;
	void* ptr = ::malloc(bytes);
	if (onAllocHandler)
		onAllocHandler(ptr, bytes, tag);
	return ptr;
}

//-----------------------------------------------------------------------------
void MemManager::Free(void* ptr) throw()
{
	::free(ptr);
	if (onFreeHandler)
		onFreeHandler(ptr);
}

//-----------------------------------------------------------------------------
void MemManager::BeginFrame()
{
}

//-----------------------------------------------------------------------------
void MemManager::EndFrame()
{
}

//-----------------------------------------------------------------------------
void MemManager::SetOnAllocHandler(OnAlloc handler)
{
	onAllocHandler = handler;
}

//-----------------------------------------------------------------------------
void MemManager::SetOnFreeHandler(OnFree handler)
{
	onFreeHandler = handler;
}

//-----------------------------------------------------------------------------
void MemManager::Exit()
{
	int tits = 1;
	sizeof(tits);
}

//-----------------------------------------------------------------------------
void MemManager::DumpHeap()
{
}

#endif // #RDE_CORE_MEMMGR_HEADER 

}
//-----------------------------------------------------------------------------

