#ifndef CORE_MEMMANAGER_H
#define CORE_MEMMANAGER_H

#include "System.h"
#include <new>

namespace rdecore
{
//-----------------------------------------------------------------------------
template<typename T> inline void DeleteIfNotNull(T* ptr)
{
    typedef char typeMustBeComplete[sizeof(T)];
    if (ptr != 0)
        delete ptr;
}
//-----------------------------------------------------------------------------
template<typename T> inline void DeleteArrayIfNotNull(T* ptr)
{
    typedef char typeMustBeComplete[sizeof(T)];
    if (ptr != 0)
        delete[] ptr;
}

/// Memory manager.
struct MemManager
{
	typedef void (*OnAlloc)(const void* address, size_t bytes, const char* tag);
	typedef void (*OnFree)(const void* address);

	static void* Alloc(size_t bytes, const char* tag = 0);
    static void Free(void* ptr) throw();

	/// For more detailed memory profiling.
	static void BeginFrame();
	static void EndFrame();

	static void SetOnAllocHandler(OnAlloc handler);
	static void SetOnFreeHandler(OnFree handler);
    
    /**
     * Deinitializes memory manager. It's forbidden to call Alloc/Fre
     * after this function has been executed.
	 * It's automatically called from atexit().
     */
    static void Exit();
    
    /**
     * Dumps heap information (textual) to file/screen/etc.
     * Exact format/destination is defined by implementation (some
     * of them may do nothing), this function should only be used 
     * for debug purposes.
     */
    static void DumpHeap();
};

}

//-----------------------------------------------------------------------------
#endif // #ifndef CORE_MEMMANAGER_H
