#include "RefCounted.h"
#include "RdeDebug.h"
#include "Threads.h"

namespace
{
	const long MAX_REF_COUNT	= 10000;
}
namespace rdecore
{

//-----------------------------------------------------------------------------
RefCounted::RefCounted(long initialRefCount)
:   m_refCount(initialRefCount)
{
    RDE_ASSERT(m_refCount >= 0);
	RDE_ASSERT(m_refCount < MAX_REF_COUNT);
}

//-----------------------------------------------------------------------------
RefCounted::RefCounted(const RefCounted& /*rhs*/)
:   m_refCount(0)
{
    /**/
}

//-----------------------------------------------------------------------------
RefCounted::~RefCounted()
{
    RDE_ASSERT(m_refCount == 0);
}

//-----------------------------------------------------------------------------
RefCounted& RefCounted::operator=(const RefCounted&)
{
    return *this;
}

//-----------------------------------------------------------------------------
void RefCounted::AddRef()
{
    RDE_ASSERT(m_refCount >= 0);
	RDE_ASSERT(m_refCount < MAX_REF_COUNT);
	threads::IncrementLong(&m_refCount);
}

//-----------------------------------------------------------------------------
void RefCounted::Release()
{
    RDE_ASSERT(m_refCount > 0);
	RDE_ASSERT(m_refCount < MAX_REF_COUNT);
	if (threads::DecrementLong(&m_refCount) <= 0)
        OnDispose();
}

//-----------------------------------------------------------------------------
long RefCounted::GetRefCount() const
{
     return m_refCount;
}

} // namespace rdecore
//-----------------------------------------------------------------------------
