#ifndef CORE_REFPOINTER_H
#define CORE_REFPOINTER_H

namespace rdecore
{
//=============================================================================
template<typename Pointed>
class RefPointer
{
public:
	enum EDontAddRef { DontAddRef };

	RefPointer():	m_ptr(0) {/**/}
	explicit RefPointer(Pointed* ptr):	m_ptr(ptr)
	{
		if (m_ptr)
			m_ptr->AddRef();
	}
	RefPointer(Pointed* ptr, EDontAddRef):	m_ptr(ptr) {/**/}
	RefPointer(const RefPointer& rhs):	m_ptr(rhs.m_ptr)
	{
		if (m_ptr)
			m_ptr->AddRef();
	}
	~RefPointer()
	{
		if (m_ptr)
			m_ptr->Release();
	}

	RefPointer& operator=(const RefPointer& rhs)
	{
		if (m_ptr)
			m_ptr->Release();
		m_ptr = rhs.m_ptr;
		if (m_ptr)
			m_ptr->AddRef();
		return *this;
	}
	RefPointer& operator=(const Pointed* ptr)
	{
		if (m_ptr != ptr)
		{
			if (m_ptr)
				m_ptr->Release();
			m_ptr = const_cast<Pointed*>(ptr);
			if (m_ptr)
				m_ptr->AddRef();
		}
		return *this;
	}

	Pointed* operator->() 
	{
		return m_ptr;
    }
    Pointed const* operator->() const 
	{
		return m_ptr;
    }
    Pointed& operator*() 
	{
		return *m_ptr;
    }
    Pointed const& operator*() const 
	{
		return *m_ptr;
    }
    bool operator!() const 
	{
		return !m_ptr;
    }
    bool operator==(const RefPointer& rhs) const
	{
      return m_ptr == rhs.m_ptr;
    }
    bool operator==(Pointed const* p) const 
	{
		return m_ptr == p;
    }
    bool operator<(const RefPointer& rhs) const 
	{
		return m_ptr < rhs.m_ptr;
    }
    bool operator<(Pointed const* p) const 
	{
		return m_ptr < p;
    }

    Pointed* GetPtr() const 
	{
		return m_ptr;
    }
private:
	Pointed*	m_ptr;
};

}

//-----------------------------------------------------------------------------
#endif // #ifndef CORE_REFPOINTER_H
