#include <UnitTest++/src/UnitTest++.h>
#include "../Global.h"

namespace
{
	using namespace rdecore;

	struct InstanceCounter
	{
		InstanceCounter()
		{
			++instanceCount;
		}
		~InstanceCounter()
		{
			--instanceCount;
		}
	    static int instanceCount;
	};
	int InstanceCounter::instanceCount = 0;

	TEST(GetReturnsValidPointer)
	{
		int* p = new int;
		Global<int> g(p);
		CHECK_EQUAL(p, g.Get());
	}
	TEST(DoesntDeleteInstanceIfNotOwned)
	{
		InstanceCounter* p(new InstanceCounter());
		{
			Global<InstanceCounter> g(p, false);
		}
		CHECK_EQUAL(1, InstanceCounter::instanceCount);
		delete p;
	}
	TEST(DeletesInstanceIfOwned)
	{
		InstanceCounter* p(new InstanceCounter());
		{
			Global<InstanceCounter> g(p);
		}
		CHECK_EQUAL(0, InstanceCounter::instanceCount);
	}
	TEST(RestoresPreviousInstanceInDestructor)
	{
		InstanceCounter* prev(new InstanceCounter());
		Global<InstanceCounter> g(prev);
		{
			InstanceCounter* curr(new InstanceCounter());
			Global<InstanceCounter> g2(curr);
		} // curr goes away, prev becomes curr
		CHECK_EQUAL(1, InstanceCounter::instanceCount);
		CHECK_EQUAL(prev, Global<InstanceCounter>::Get());
	}

} // namespace
