#include <UnitTest++/src/UnitTest++.h>
#include "../RefCounted.h"
#include "../RefPointer.h"

namespace
{
	using rdecore::RefPointer;
	using rdecore::RefCounted;

	TEST(DefaultCtorGivesNullPtr)
	{
		RefPointer<RefCounted>	ptr;
		CHECK_EQUAL((RefCounted*)0, ptr.GetPtr());
		CHECK(!ptr);
		CHECK(ptr == 0);
	}
	TEST(CtorFromNakedPointer)
	{
		RefCounted* r(new RefCounted());
		CHECK_EQUAL(0, r->GetRefCount());
		{
			RefPointer<RefCounted>	ptr(r);
			CHECK(!!ptr);
			CHECK(!(ptr == 0));
			CHECK_EQUAL(1, r->GetRefCount());
		}
	}
	TEST(AssignmentFromNakedPointer)
	{
		RefCounted* r(new RefCounted());
		CHECK_EQUAL(0, r->GetRefCount());
		{
			RefPointer<RefCounted>	ptr;
			ptr = r;
			CHECK(!!ptr);
			CHECK(!(ptr == 0));
			CHECK_EQUAL(1, r->GetRefCount());
		}
	}
	TEST(CtorFromNakedPointerDontAddRef)
	{
		RefCounted* r(new RefCounted());
		CHECK_EQUAL(0, r->GetRefCount());
		{
			RefPointer<RefCounted>	ptr(r, RefPointer<RefCounted>::DontAddRef);
			CHECK_EQUAL(0, r->GetRefCount());
			r->AddRef();
		}
	}
	TEST(CopyCtor)
	{
		RefCounted* r(new RefCounted());
		RefPointer<RefCounted>	ptr(r);
		RefPointer<RefCounted>	ptr2(ptr);
		CHECK_EQUAL(2, r->GetRefCount());
		CHECK(ptr == ptr2);
	}
	TEST(AssignmentOp)
	{
		RefCounted* r(new RefCounted());
		RefPointer<RefCounted>	ptr(r);
		RefPointer<RefCounted>	ptr2;
		ptr2 = ptr;
		CHECK_EQUAL(2, r->GetRefCount());
		CHECK(ptr == ptr2);
	}
}

