#ifndef CORE_MSVC_CONFIG_H
#define CORE_MSVC_CONFIG_H

// Compiler detection
#if !defined (_MSC_VER)
#       error "MSVC C++ compiler required!"
#endif

//-----------------------------------------------------------------------------
// Platform/compiler definitions
//-----------------------------------------------------------------------------

#define RDE_PLATFORM_WIN			1                  // Platform = WIN
#define RDE_COMPILER_GNU			0                  // GNU family...
#define RDE_COMPILER_MSVC			1	

// Info strings (compiler/platform info)
#define RDE_COMPILER_NAME        "MSVC"
#define RDE_PLATFORM_NAME        "Windows"

// Win is little endian
#define RDE_LITTLE_ENDIAN        1
#undef RDE_BIG_ENDIAN

// 80x86 CPU assumed
#define RDE_CPU_NAME             "80x86"

#ifdef _DEBUG
#	undef RDE_DEBUG
#	define RDE_DEBUG	1
#endif

//-----------------------------------------------------------------------------
// Disable over sensitive warnings
//-----------------------------------------------------------------------------

// std::size_t -> unsigned long conversion
#pragma warning(disable: 4267)
// warning C4127: conditional expression is constant
// (explodes on constructs like while (true) {..}
#pragma warning(disable: 4127)
// warning C4511: copy constructor could not be generated
#pragma warning(disable: 4511)
// warning C4512: assignment operator could not be generated
#pragma warning(disable: 4512)

//-----------------------------------------------------------------------------
// Map compiler directives to unified system
//-----------------------------------------------------------------------------

#define RDE_FORCEINLINE  __forceinline			// Force code to be inline
// Import from DLL
#define RDE_DLL_IMPORT   __declspec(dllimport)
// Export to DLL
#define RDE_DLL_EXPORT   __declspec(dllexport)
#define RDE_CDECL        __cdecl                // Standard C function
#define RDE_STDCALL      __stdcall              // Standard calling convention


//-----------------------------------------------------------------------------
// Compiler 'capabilities'
//-----------------------------------------------------------------------------

#define RDE_COMPILER_HAS_MEMBER_TEMPLATES               1
#define RDE_COMPILER_HAS_MEMBER_TEMPLATES_OUTSIDE       1
#define RDE_COMPILER_HAS_EXPLICIT_TMPL_SPECIALIZATION   1
#define RDE_COMPILER_HAS_TMPL_INST                      1

// Even if it has, I have some problems so I disable it!!!!
#define RDE_COMPILER_HAS_MEMBER_TEMPLATE_FRIENDS		0
# define RDE_COMPILER_HAS_NOTHROW						1

#define RDE_COMPILER_HAS_DLL_SUPPORT                    1

/// 8-bit unsigned
typedef unsigned char   uint8_t;
/// 16-bit unsigned
typedef unsigned short  uint16_t;
/// 32-bit unsigned
typedef unsigned long   uint32_t;

//-----------------------------------------------------------------------------
// System specific includes
//-----------------------------------------------------------------------------

#define RDE_SYS_TIMER_HEADER			"win32/Timer.h"

//-----------------------------------------------------------------------------
#endif  // CORE_MSVC_CONFIG.H
